@@ -2,7 +2,7 @@ use std::sync::Arc;
 
 use gpui::WindowBackgroundAppearance;
 
-use crate::prelude::*;
+use crate::AccentColors;
 
 use crate::{
     default_color_scales,
@@ -23,21 +23,7 @@ fn zed_pro_daylight() -> Theme {
             status: StatusColors::light(),
             player: PlayerColors::light(),
             syntax: Arc::new(SyntaxTheme::default()),
-            accents: vec![
-                blue().light().step_9(),
-                orange().light().step_9(),
-                pink().light().step_9(),
-                lime().light().step_9(),
-                purple().light().step_9(),
-                amber().light().step_9(),
-                jade().light().step_9(),
-                tomato().light().step_9(),
-                cyan().light().step_9(),
-                gold().light().step_9(),
-                grass().light().step_9(),
-                indigo().light().step_9(),
-                iris().light().step_9(),
-            ],
+            accents: AccentColors::light(),
         },
     }
 }
@@ -54,21 +40,7 @@ pub(crate) fn zed_pro_moonlight() -> Theme {
             status: StatusColors::dark(),
             player: PlayerColors::dark(),
             syntax: Arc::new(SyntaxTheme::default()),
-            accents: vec![
-                blue().dark().step_9(),
-                orange().dark().step_9(),
-                pink().dark().step_9(),
-                lime().dark().step_9(),
-                purple().dark().step_9(),
-                amber().dark().step_9(),
-                jade().dark().step_9(),
-                tomato().dark().step_9(),
-                cyan().dark().step_9(),
-                gold().dark().step_9(),
-                grass().dark().step_9(),
-                indigo().dark().step_9(),
-                iris().dark().step_9(),
-            ],
+            accents: AccentColors::dark(),
         },
     }
 }
