@@ -77,9 +77,11 @@ pub enum IconName {
     Ai,
     ArrowCircle,
     ArrowDown,
+    ArrowDownFromLine,
     ArrowLeft,
     ArrowRight,
     ArrowUp,
+    ArrowUpFromLine,
     ArrowUpRight,
     AtSign,
     AudioOff,
@@ -161,7 +163,10 @@ pub enum IconName {
     ReplaceAll,
     ReplaceNext,
     ReplyArrowRight,
+    Rerun,
     Return,
+    Reveal,
+    Save,
     Screen,
     SelectAll,
     Server,
@@ -180,6 +185,7 @@ pub enum IconName {
     Tab,
     Terminal,
     Trash,
+    TriangleRight,
     Update,
     WholeWord,
     XCircle,
@@ -193,6 +199,7 @@ impl IconName {
             IconName::Ai => "icons/ai.svg",
             IconName::ArrowCircle => "icons/arrow_circle.svg",
             IconName::ArrowDown => "icons/arrow_down.svg",
+            IconName::ArrowDownFromLine => "icons/arrow_down_from_line.svg",
             IconName::ArrowLeft => "icons/arrow_left.svg",
             IconName::ArrowRight => "icons/arrow_right.svg",
             IconName::ArrowUp => "icons/arrow_up.svg",
@@ -274,10 +281,13 @@ impl IconName {
             IconName::Quote => "icons/quote.svg",
             IconName::Regex => "icons/regex.svg",
             IconName::Replace => "icons/replace.svg",
+            IconName::Reveal => "icons/reveal.svg",
             IconName::ReplaceAll => "icons/replace_all.svg",
             IconName::ReplaceNext => "icons/replace_next.svg",
             IconName::ReplyArrowRight => "icons/reply_arrow_right.svg",
+            IconName::Rerun => "icons/rerun.svg",
             IconName::Return => "icons/return.svg",
+            IconName::Save => "icons/save.svg",
             IconName::Screen => "icons/desktop.svg",
             IconName::SelectAll => "icons/select_all.svg",
             IconName::Server => "icons/server.svg",
@@ -296,11 +306,13 @@ impl IconName {
             IconName::Tab => "icons/tab.svg",
             IconName::Terminal => "icons/terminal.svg",
             IconName::Trash => "icons/trash.svg",
+            IconName::TriangleRight => "icons/triangle_right.svg",
             IconName::Update => "icons/update.svg",
             IconName::WholeWord => "icons/word_search.svg",
             IconName::XCircle => "icons/error.svg",
             IconName::ZedAssistant => "icons/zed_assistant.svg",
             IconName::ZedXCopilot => "icons/zed_x_copilot.svg",
+            IconName::ArrowUpFromLine => "icons/arrow_up_from_line.svg",
         }
     }
 }
